from __future__ import annotations

from pygismeteo_base import models, types
from pygismeteo_base.current import CurrentBase

from ._http import RequestsClient


class Current(CurrentBase[RequestsClient]):
    """Текущая погода."""

    __slots__ = ()

    def by_coordinates(
        self, latitude: types.Latitude, longitude: types.Longitude
    ) -> models.current.Model:
        """По координатам.

        Args:
            latitude (-90 ≤ float ≤ 90):
                Широта.
            longitude (-180 ≤ float ≤ 180):
                Долгота.
        """
        url, params = self._get_params_by_coordinates(latitude, longitude)
        return self._get_result(url, params=params)

    def by_id(self, id: types.LocalityID) -> models.current.Model:  # noqa: A002
        """По ID географического объекта.

        Args:
            id (int ≥ 1):
                ID географического объекта. Получить можно через поиск.
        """
        url, params = self._get_params_by_id(id)
        return self._get_result(url, params=params)

    def _get_result(
        self, url: str, *, params: types.Params = None
    ) -> models.current.Model:
        response = self._session.get_response(url, params=params)
        model = models.current.Response.model_validate_json(response)
        return model.response



"""Обёртка для Gismeteo API."""

from __future__ import annotations

from pygismeteo_base import models

from . import types
from ._gismeteo import Gismeteo

__all__ = ("Gismeteo", "models", "types")



from __future__ import annotations

from typing import Mapping, Optional

from pygismeteo_base import types
from pygismeteo_base.http import BaseHttpClient
from requests import Response, Session


class RequestsClient(BaseHttpClient[Session]):
    __slots__ = ()

    def get_response(
        self, endpoint: str, *, params: types.Params = None
    ) -> str:
        params, headers = self._get_params_and_headers(params)
        if isinstance(self.session, Session):
            response = self._fetch(
                endpoint, params=params, headers=headers, session=self.session
            )
        else:
            with Session() as session:
                response = self._fetch(
                    endpoint, params=params, headers=headers, session=session
                )
        response.raise_for_status()
        return response.text

    def _fetch(  # noqa: PLR6301
        self,
        endpoint: str,
        *,
        params: Optional[Mapping[str, str]],
        headers: Mapping[str, str],
        session: Session,
    ) -> Response:
        with session.get(
            f"https://api.gismeteo.net/v2/{endpoint}/",
            params=params,
            headers=headers,
        ) as response:
            return response



from __future__ import annotations

from ipaddress import IPv4Address
from typing import List, Union

from pygismeteo_base import models, types
from pygismeteo_base.search import SearchBase
from typing_extensions import Literal, overload

from ._http import RequestsClient


class Search(SearchBase[RequestsClient]):
    """Поиск."""

    __slots__ = ()

    @overload
    def by_coordinates(
        self,
        latitude: types.Latitude,
        longitude: types.Longitude,
        limit: types.SearchLimit,
        *,
        as_list: Literal[True] = ...,
    ) -> List[models.search_by_coordinates.ModelItem]: ...

    @overload
    def by_coordinates(
        self,
        latitude: types.Latitude,
        longitude: types.Longitude,
        limit: types.SearchLimit,
        *,
        as_list: Literal[False],
    ) -> models.search_by_coordinates.Model: ...

    @overload
    def by_coordinates(
        self,
        latitude: types.Latitude,
        longitude: types.Longitude,
        limit: types.SearchLimit,
        *,
        as_list: bool,
    ) -> Union[
        List[models.search_by_coordinates.ModelItem],
        models.search_by_coordinates.Model,
    ]: ...

    def by_coordinates(
        self,
        latitude: types.Latitude,
        longitude: types.Longitude,
        limit: types.SearchLimit,
        *,
        as_list: bool = True,
    ) -> Union[
        List[models.search_by_coordinates.ModelItem],
        models.search_by_coordinates.Model,
    ]:
        """По координатам.

        Args:
            latitude (-90 ≤ float ≤ 90):
                Широта.
            longitude (-180 ≤ float ≤ 180):
                Долгота.
            limit (1 ≤ int ≤ 36):
                Ограничение количества результатов.
            as_list (bool):
                Вернуть Model.root (list[ModelItem]) вместо Model.
                По умолчанию True.
        """
        params = self._get_params_by_coordinates(
            latitude=latitude, longitude=longitude, limit=limit
        )
        response = self._get_response(params)
        model = models.search_by_coordinates.Response.model_validate_json(
            response
        )
        return model.response.root if as_list else model.response

    def by_ip(self, ip: Union[IPv4Address, str]) -> models.search_by_ip.Model:
        """По IPv4-адресу.

        Args:
            ip (ipaddress.IPv4Address | str):
                IPv4-адрес.
        """
        params = self._get_params_by_ip(ip)
        response = self._get_response(params)
        model = models.search_by_ip.Response.model_validate_json(response)
        return model.response

    @overload
    def by_query(
        self, query: str, *, as_list: Literal[True] = ...
    ) -> List[models.search_by_query.ModelItem]: ...

    @overload
    def by_query(
        self, query: str, *, as_list: Literal[False]
    ) -> models.search_by_query.Model: ...

    @overload
    def by_query(
        self, query: str, *, as_list: bool
    ) -> Union[
        List[models.search_by_query.ModelItem], models.search_by_query.Model
    ]: ...

    def by_query(
        self, query: str, *, as_list: bool = True
    ) -> Union[
        List[models.search_by_query.ModelItem], models.search_by_query.Model
    ]:
        """По строке.

        Args:
            query (str):
                Город, район, область, страна или аэропорт.
            as_list (bool):
                Вернуть Model.root (list[ModelItem]) вместо Model.
                По умолчанию True.
        """
        params = self._get_params_by_query(query)
        response = self._get_response(params)
        model = models.search_by_query.Response.model_validate_json(response)
        return model.response.items.root if as_list else model.response.items

    def _get_response(self, params: types.Params) -> str:
        return self._session.get_response(self._endpoint(), params=params)



from __future__ import annotations

from typing import Generic, List, Union

from pygismeteo_base import models, types
from pygismeteo_base.step_n import mixins
from pygismeteo_base.step_n.abc import StepNABC
from typing_extensions import Literal, overload

from ._http import RequestsClient


class StepN(
    Generic[types.TStepNDays, types.TStepNModel, types.TStepNModelItem],
    StepNABC[RequestsClient],
):
    __slots__ = ()

    @overload
    def by_coordinates(
        self,
        latitude: types.Latitude,
        longitude: types.Longitude,
        days: types.TStepNDays,
        *,
        as_list: Literal[True] = ...,
    ) -> List[types.TStepNModelItem]: ...

    @overload
    def by_coordinates(
        self,
        latitude: types.Latitude,
        longitude: types.Longitude,
        days: types.TStepNDays,
        *,
        as_list: Literal[False],
    ) -> types.TStepNModel: ...

    @overload
    def by_coordinates(
        self,
        latitude: types.Latitude,
        longitude: types.Longitude,
        days: types.TStepNDays,
        *,
        as_list: bool,
    ) -> Union[List[types.TStepNModelItem], types.TStepNModel]: ...

    def by_coordinates(
        self,
        latitude: types.Latitude,
        longitude: types.Longitude,
        days: types.TStepNDays,
        *,
        as_list: bool = True,
    ) -> Union[List[types.TStepNModelItem], types.TStepNModel]:
        """По координатам.

        Args:
            latitude (-90 ≤ float ≤ 90):
                Широта.
            longitude (-180 ≤ float ≤ 180):
                Долгота.
            days (
                step3:  1 ≤ int ≤ 10,
                step6:  3 ≤ int ≤ 10,
                step24: 3 ≤ int ≤ 10,
            ):
                Количество дней.
            as_list (bool):
                Вернуть Model.root (list[ModelItem]) вместо Model.
                По умолчанию True.
        """
        url, params = self._get_params_by_coordinates(
            latitude, longitude, days=days
        )
        return self._get_result(url, params=params, as_list=as_list)

    @overload
    def by_id(
        self,
        id: types.LocalityID,  # noqa: A002
        days: types.TStepNDays,
        *,
        as_list: Literal[True] = ...,
    ) -> List[types.TStepNModelItem]: ...

    @overload
    def by_id(
        self,
        id: types.LocalityID,  # noqa: A002
        days: types.TStepNDays,
        *,
        as_list: Literal[False],
    ) -> types.TStepNModel: ...

    @overload
    def by_id(
        self,
        id: types.LocalityID,  # noqa: A002
        days: types.TStepNDays,
        *,
        as_list: bool,
    ) -> Union[List[types.TStepNModelItem], types.TStepNModel]: ...

    def by_id(
        self,
        id: types.LocalityID,  # noqa: A002
        days: types.TStepNDays,
        *,
        as_list: bool = True,
    ) -> Union[List[types.TStepNModelItem], types.TStepNModel]:
        """По ID географического объекта.

        Args:
            id (int ≥ 1):
                ID географического объекта. Получить можно через поиск.
            days (
                step3:  1 ≤ int ≤ 10,
                step6:  3 ≤ int ≤ 10,
                step24: 3 ≤ int ≤ 10,
            ):
                Количество дней.
            as_list (bool):
                Вернуть Model.root (list[ModelItem]) вместо Model.
                По умолчанию True.
        """
        url, params = self._get_params_by_id(id, days=days)
        return self._get_result(url, params=params, as_list=as_list)

    @overload
    def _get_result(
        self, url: str, *, params: types.Params, as_list: Literal[True]
    ) -> List[types.TStepNModelItem]: ...

    @overload
    def _get_result(
        self, url: str, *, params: types.Params, as_list: Literal[False]
    ) -> types.TStepNModel: ...

    @overload
    def _get_result(
        self, url: str, *, params: types.Params, as_list: bool
    ) -> Union[List[types.TStepNModelItem], types.TStepNModel]: ...

    def _get_result(
        self, url: str, *, params: types.Params, as_list: bool
    ) -> Union[List[types.TStepNModelItem], types.TStepNModel]:
        response = self._session.get_response(url, params=params)
        model = self._model().model_validate_json(response)
        return (
            model.response.root  # type: ignore[return-value]
            if as_list
            else model.response
        )


class Step3(
    mixins.Step3Mixin,
    StepN[types.Step3Days, models.step3.Model, models.step3.ModelItem],
):
    """Погода с шагом 3 часа."""

    __slots__ = ()


class Step6(
    mixins.Step6Mixin,
    StepN[types.Step6Days, models.step6.Model, models.step6.ModelItem],
):
    """Погода с шагом 6 часов."""

    __slots__ = ()


class Step24(
    mixins.Step24Mixin,
    StepN[types.Step24Days, models.step24.Model, models.step24.ModelItem],
):
    """Погода с шагом 24 часа."""

    __slots__ = ()



from __future__ import annotations

from typing import Final, Optional

from pygismeteo_base.types import Lang
from pygismeteo_base.validators import Settings
from requests import Session
from typing_extensions import final

from ._current import Current
from ._http import RequestsClient
from ._search import Search
from ._step_n import Step3, Step6, Step24


@final
class Gismeteo:
    """Обёртка для Gismeteo API."""

    __slots__ = (
        "_session",
        "_settings",
        "current",
        "search",
        "step3",
        "step6",
        "step24",
    )

    def __init__(
        self,
        *,
        lang: Optional[Lang] = None,
        session: Optional[Session] = None,
        token: str,
    ) -> None:
        """Обёртка для Gismeteo API.

        Args:
            lang:
                Язык. По умолчанию "ru".
            session:
                Экземпляр requests.Session.
                По умолчанию для каждого запроса создаётся новый экземпляр.
            token:
                X-Gismeteo-Token.
                Запросить можно по электронной почте b2b@gismeteo.ru.
        """
        self._settings = Settings(lang=lang, token=token)
        self._session = RequestsClient(session, self._settings)
        self.current: Final = Current(self._session)
        """Текущая погода."""
        self.search: Final = Search(self._session)
        """Поиск."""
        self.step3: Final = Step3(self._session)
        """Погода с шагом 3 часа."""
        self.step6: Final = Step6(self._session)
        """Погода с шагом 6 часов."""
        self.step24: Final = Step24(self._session)
        """Погода с шагом 24 часа."""

    @property
    def lang(self) -> Optional[Lang]:
        """Язык."""
        return self._settings.lang

    @lang.setter
    def lang(self, lang: Optional[Lang]) -> None:
        self._settings.lang = lang

    @property
    def session(self) -> Optional[Session]:
        return self._session.session

    @session.setter
    def session(self, session: Optional[Session]) -> None:
        self._session.session = session

    @property
    def token(self) -> str:
        """X-Gismeteo-Token."""
        return self._settings.token

    @token.setter
    def token(self, token: str) -> None:
        self._settings.token = token



"""Может пригодиться для типизации кода, использующего библиотеку."""

from __future__ import annotations

from pygismeteo_base.types import (
    Lang,
    Latitude,
    LocalityID,
    Longitude,
    SearchLimit,
    Step3Days,
    Step6Days,
    Step24Days,
)

from ._current import Current
from ._search import Search
from ._step_n import Step3, Step6, Step24

__all__ = (
    "Current",
    "Lang",
    "Latitude",
    "LocalityID",
    "Longitude",
    "Search",
    "SearchLimit",
    "Step3",
    "Step3Days",
    "Step6",
    "Step6Days",
    "Step24",
    "Step24Days",
)
